#ifndef _KEY_H_
#define _KEY_H_

#define NUM_KEYS	10

#define KEY_UP		0x1
#define KEY_DOWN	0x2
#define KEY_LEFT	0x4
#define KEY_RIGHT	0x8
#define KEY_TRIG1	0x10
#define KEY_TRIG2	0x20
#define KEY_TRIG3	0x40
#define KEY_TRIG4	0x80
#define KEY_PAUSE	0x100
#define KEY_QUIT	0x200

void KEY_init( void );
void KEY_add( Display *d, int key, char *keysym_name );
int KEY_change( int keycode, int state );
int KEY_read( void );

#endif
