/*
** Falling Tower KAI
**	Copyright (c) 2000	ROYALPANDA	All rights reserved
**
**	May 25, 2000 Ver 1.0
*/

#include<stdio.h>

char *DefaultResources[] = {

"XJumpX*keyUp:		Up",
"XJumpX*keyDown:	Down",
"XJumpX*keyLeft:	Left",
"XJumpX*keyRight:	Right",
"XJumpX*keyTrig1:	space",
"XJumpX*keyTrig2:	n",
"XJumpX*keyTrig3:	x",
"XJumpX*keyTrig4:	c",
"XJumpX*keyPause:	p",

"XJumpX*font                     : -adobe-*-*-*-*-*-24-*-*-*-*-*-*-*",
"XJumpX*foreground               : white",
"XJumpX*background               : black",
"XJumpX*borderColor              : white",
"XJumpX*defaultDistance          : 16",
"XJumpX*borderWidth              : 0",

"XJumpX*title.horizDistance      : 64",

"XJumpX*scLabel.label            : Floor",
"XJumpX*scLabel.fromHoriz        : title",
"XJumpX*scLabel.vertDistance     : 48",
"XJumpX*scLabel.horizDistance    : 32",

"XJumpX*score.fromVert           : scLabel",
"XJumpX*score.fromHoriz          : title",
"XJumpX*score.vertDistance       : 4",
"XJumpX*score.horizDistance      : 64",

"XJumpX*gameover.label           : Game Over",
"XJumpX*gameover.fromVert        : board",
"XJumpX*gameover.horizDistance   : 200",
"XJumpX*gameover.vertDistance    : 96",
"XJumpX*gameover.foreground      : white",
"XJumpX*gameover.background      : blue",
"XJumpX*gameover.borderColor     : navy",
"XJumpX*gameover.borderWidth     : 4",

"XJumpX*pause.label              : Pause",
"XJumpX*pause.fromVert           : board",
"XJumpX*pause.horizDistance      : 224",
"XJumpX*pause.vertDistance       : 96",
"XJumpX*pause.foreground         : white",
"XJumpX*pause.background         : blue",
"XJumpX*pause.borderColor        : navy",
"XJumpX*pause.borderWidth        : 4",

"XJumpX*scr.fromVert             : board",
"XJumpX*scr.borderWidth          : 0",

"XJumpX*record_v.fromVert        : board",
"XJumpX*record_v.allowVert       : True",
"XJumpX*record_v.allowHoriz      : True",
"XJumpX*record_v*borderWidth     : 1",

"XJumpX*record.font		: -*-*-*-*-*-*-14-*-*-*-c-*-*-*",

"XJumpX*message.fromVert	: scr",
"XJumpX*message.foreground	: green",
"XJumpX*message.font		: -*-fixed-*-*-*-*-20-*-*-*-c-*-*-*",
"XJumpX*message.justify		: center",
NULL};
